<?php

namespace Teknisa\FIN\Modules\ContaPagar\Services;

use Doctrine\ORM\EntityManager;
use Teknisa\Libs\Util\NewCode;
use Teknisa\Libs\Util\DateUtil;
use Teknisa\Model\Financeiro\Entities\Logboletorempag;
use Teknisa\Model\Financeiro\Entities\Boletpagar;
use Teknisa\Model\Financeiro\Entities\Cnabpag;
use Teknisa\FIN\Modules\ContaPagar\Factory\RemessaTituloFactory ;
use Teknisa\FIN\Modules\ContaPagar\Factory\TitulosAPagarFactory;
use Teknisa\FIN\Modules\ContaPagar\Services\Remessa\GeraCnabService;
//use Zeedhi\FileServerAPI\API;

/**
 * Class RemessaTituloService
 * @package Teknisa\FIN\Modules\ContaPagar\Services
 */
class RemessaTituloService {

    protected $entityManager;
    protected $remessaTituloFactory;
    protected $titulosAPagarFactory;
    protected $newCode;
    protected $server;

    const CUSTOM_FOLDER   = 'erp/modules/fin/modules/contaPagar/cnab/remessa';
    const FILE_FIELD_NAME = 'LOCAL_ARQUIVO';
    const ID_STATUS_BOLETO_ENVIADO     = '10';
    const CODIGO_COMPENSACAO_SANTANDER = '033';
    const CODIGO_COMPENSACAO_ITAU      = '341';
    const CODIGO_COMPENSACAO_BRADESCO  = "237";
    const CODIGO_COMPENSACAO_CAIXA     = "104";
    const CODIGO_COMPENSACAO_BRASIL    = '001';
    const CODIGO_COMPENSACAO_SICREDI   = '748';
    const CODIGO_COMPENSACAO_SICOOB    = '756';
    const ID_BOLETO_PAGAR_ALTERADO     = '1';
    const IDTIPCNABPAG_REMESSA         = '0';
    const PREFIXO_NOME_ARQUIVO         = 'P';

    /**
     * @param EntityManager $entityManager
     * @param RemessaTituloFactory $remessaTituloFactory
     * @param TitulosAPagarFactory $titulosAPagarFactory
     * @param NewCode $newCode
     * @param Server $server
     */
    public function __construct(EntityManager $entityManager, RemessaTituloFactory $remessaTituloFactory, TitulosAPagarFactory $titulosAPagarFactory, NewCode $newCode, $server)
    {
        $this->entityManager        = $entityManager;
        $this->remessaTituloFactory  = $remessaTituloFactory;
        $this->titulosAPagarFactory = $titulosAPagarFactory;
        $this->newCode = $newCode;
        $this->server  = $server;
    }

    /**
     * Gera Remessa Pagar
     * @param $nrorg
     * @param $cdfilial
     * @param $remessa
     * @param $boletos
     * @return null|string
     * @throws \Exception
     */
    public function geraRemessaPagar(
        $nrorg, $cdfilial, $remessa, $boletos
    ){
        $connection = $this->entityManager->getConnection();
        $connection->beginTransaction();
        try {
            // Gera Novo Código CNAB Pagar
            $nrcnabpag = $this->geraNovoCodigo($nrorg, 'CNABPAG'.$remessa['CDEMPRESA']);

            // Caminho do Arquivo
            $dsarqcnabpag = $remessa['DSARQCNABPAG'];
            
            // Nome do Arquivo
            $nmarqcnabpag = self::PREFIXO_NOME_ARQUIVO.$nrorg.$remessa['CDEMPRESA'].$nrcnabpag;

            // Gera Novo Código Sequencial Remessa
            $nrseqremessapag = $this->geraNovoCodigo($nrorg, 'SEQREMESSAPAG'.$remessa['CDREDUCONT']);

            // Formata Data CNAB
            $dtcnabpag = self::formataDataHora($remessa['DTCNABPAG']);

            $autorizaAgendamento = array_key_exists('IDAGENREMCNABPAG', $remessa);

            // Autoriza Agendamento
            $idagenremcnabpag = array_key_exists('IDAGENREMCNABPAG', $remessa)?$remessa['IDAGENREMCNABPAG']:null;
            $cdfilialrem      = array_key_exists('CDFILIAL', $remessa)?$remessa['CDFILIAL']:null;
            
            // Tipo de Remessa
            $idtpremcnabpag = array_key_exists('IDTPREMCNABPAG', $remessa)?$remessa['IDTPREMCNABPAG']:null;
            
            // Pré Negociado
            $idtpremcnabpag = array_key_exists('IDPRENEGOCIADO', $remessa)?$remessa['IDPRENEGOCIADO']?$remessa['IDPRENEGOCIADO']:'N':'N';
            
            // Insere Remessa Pagar
            /** @var Cnabpag $cnabpag */
            $cnabpag = $this->remessaTituloFactory->insereCnabPag(
                $nrorg, $remessa['CDEMPRESA'], $nrcnabpag, $remessa['CDOPERADOR'],
                $dtcnabpag, self::IDTIPCNABPAG_REMESSA, $remessa['CDREDUCONT'],
                $remessa['CDBANCO'], $remessa['IDOPECNABPAG'],
                $remessa['QTBOLCNABPAG'], $remessa['VRTOTALCNABPAG'], $nrseqremessapag,
                $dsarqcnabpag, $cdfilialrem, $idagenremcnabpag, $idtpremcnabpag,
                $idtpremcnabpag
            );

            $this->entityManager->persist($cnabpag);
            $this->entityManager->flush();

            // Atualiza Boletos
            $this->atualizaBoletos($nrorg, $cdfilial, $boletos, $cnabpag);

            // Caminho do Arquivo + Nome do Arquivo
            $dsarqcnabpag .= $nmarqcnabpag;

            // Verifica Banco Cnab Pagar
            self::verificaBancoCnabPag(
                $boletos, $cnabpag, $dsarqcnabpag, $autorizaAgendamento
            );

            // Caminho do Arquivo + Nome do Arquivo + Extenção
            $dsarqcnabpag .= '.REM';

            // Atualiza Local Arquivo CNAB Pagar
            self::atualizaLocalArqCnabPag($nrorg, $cnabpag->getCdempresa(), $nrcnabpag, $dsarqcnabpag);

            $this->entityManager->flush();
            $connection->commit();

            return $dsarqcnabpag;
        }catch(\Exception $e){
            $connection->rollback();
            throw $e;
            return null;
        }
    }

    /**
     * Atualiza Boletos Pagar
     * @param $nrorg
     * @param $cdfilial
     * @param $boletos
     * @param Cnabpag $cnabpag
     * @throws \Exception
     */
    public function atualizaBoletos($nrorg, $cdfilial, $boletos, Cnabpag $cnabpag){
        try {
            // Percorre Boletos Pagar
            /** @var $boleto Boletpagar */
            foreach ($boletos as $boleto) {

                // Busca Boleto Pagar
                /** @var Boletpagar $boletpagar */
                $boletpagar = $this->remessaTituloFactory->buscaBoletpagar(
                    $boleto['NRORG'], $boleto['CDFILIAL'], $boleto['NRLANBOLPAG']
                );

                // Atualiza Bolet Pagar
                $boletpagar = $this->remessaTituloFactory->atualizaBoletpagar(
                    $boletpagar, $cnabpag->getNrcnabpag(), self::ID_STATUS_BOLETO_ENVIADO
                );

                $this->entityManager->persist($boletpagar);
                $this->entityManager->flush();
                
                /** @var Logboletorempag $logboletorempag */
                $logboletorempag = $this->remessaTituloFactory->buscaLogboletorempag(
                    $nrorg, $boleto['CDFILIAL'], $boleto['NRLANBOLPAG'], $boleto['CDEMPRESA'], $cnabpag->getNrcnabpag()
                );
                
                if(!$logboletorempag) {
                    // Insere Log de Atualização Boleto Remessa Pagar
                    /** @var Logboletorempag $logboletorempag */
                    $logboletorempag = $this->remessaTituloFactory->insereLogBoletoRem(
                        $nrorg, $boleto['CDFILIAL'], $boleto['NRLANBOLPAG'], $cnabpag->getNrcnabpag(),
                        $boleto['CDEMPRESA'], $cnabpag->getCdoperador()
                    );
                    $this->entityManager->persist($logboletorempag);
                }
                
                $this->entityManager->flush();
            }
        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * Atualiza Local Arquivo CNAB Pagar[File Server]
     * @param $nrorg
     * @param $cdempresa
     * @param $nrcnabpag
     * @param $dsarqcnabpag
     * @return Cnabpag
     */
    public function atualizaLocalArqCnabPag(
        $nrorg, $cdempresa, $nrcnabpag, $dsarqcnabpag
    ){

        /** @var Cnabpag $cnabpag */
        $cnabpag = $this->remessaTituloFactory->buscaCnabpag(
            $nrorg, $cdempresa, $nrcnabpag
        );

        /** @var Cnabpag $cnabpag */
        $cnabpag = $this->remessaTituloFactory->atualizaCnabpag(
            $cnabpag, $dsarqcnabpag
        );

        return $cnabpag;
    }

    /**
     * Verifica Banco CNAB Pagar
     * @param $boletos
     * @param Cnabpag $cnabpag
     * @param $caminho
     * @param $autorizaAgendamento
     */
    public static function verificaBancoCnabPag(
        $boletos, Cnabpag $cnabpag, $caminho, $autorizaAgendamento
    ){
        $codigoCompensacao = $cnabpag->getCdbanco();
        if($codigoCompensacao === self::CODIGO_COMPENSACAO_SANTANDER){
            self::geraCnabSantander(
                $boletos, $cnabpag, $caminho, $autorizaAgendamento
            );
        }
        elseif ($codigoCompensacao === self::CODIGO_COMPENSACAO_ITAU) {
            self::geraCnabItau(
                $boletos, $cnabpag, $caminho, $autorizaAgendamento
            );
        }
        elseif ($codigoCompensacao === self::CODIGO_COMPENSACAO_CAIXA) {
            self::geraCnabCaixa(
                $boletos, $cnabpag, $caminho, $autorizaAgendamento
            );
        }
        elseif ($codigoCompensacao === self::CODIGO_COMPENSACAO_BRADESCO) {
            self::geraCnabBradesco(
                $boletos, $cnabpag, $caminho, $autorizaAgendamento
            );
        }
        elseif ($codigoCompensacao === self::CODIGO_COMPENSACAO_BRASIL) {
            self::geraCnabBrasil(
                $boletos, $cnabpag, $caminho, $autorizaAgendamento
            );
        }
        elseif ($codigoCompensacao === self::CODIGO_COMPENSACAO_SICREDI) {
            self::geraCnabSicredi(
                $boletos, $cnabpag, $caminho, $autorizaAgendamento
            );
        }
        elseif ($codigoCompensacao === self::CODIGO_COMPENSACAO_SICOOB) {
            self::geraCnabSicoob(
                $boletos, $cnabpag, $caminho, $autorizaAgendamento
            );
        }
    }

    /**
     * Gera CNAB Santander
     * @param $boletos
     * @param Cnabpag $cnabpag
     * @param $caminho
     * @param bool $autorizaAgendamento
     * @throws \Exception
     */
    public static function geraCnabSantander(
        $boletos, Cnabpag $cnabpag, $caminho, $autorizaAgendamento
    ){
        $geraCnab = GeraCnabService::factory($cnabpag);
        $geraCnab->geraArquivo($caminho, $boletos, $autorizaAgendamento, $cnabpag->getIdopecnabpag());
    }

    /**
     * Gera CNAB Itau
     * @param $boletos
     * @param Cnabpag $cnabpag
     * @param $caminho
     * @param bool $autorizaAgendamento
     * @throws \Exception
     */
    public static function geraCnabItau(
        $boletos, Cnabpag $cnabpag, $caminho, $autorizaAgendamento
    ){
        $geraCnab = GeraCnabService::factory($cnabpag);
        $geraCnab->geraArquivo($caminho, $boletos, $autorizaAgendamento, $cnabpag->getIdopecnabpag());
    }

    /**
     * Gera CNAB Sicredi
     * @param $boletos
     * @param Cnabpag $cnabpag
     * @param $caminho
     * @param bool $autorizaAgendamento
     * @throws \Exception
     */
    public static function geraCnabSicredi(
        $boletos, Cnabpag $cnabpag, $caminho, $autorizaAgendamento
    ){
        $geraCnab = GeraCnabService::factory($cnabpag);
        $geraCnab->geraArquivo($caminho, $boletos, $autorizaAgendamento, $cnabpag->getIdopecnabpag());
    }

    /**
     * Gera CNAB Sicood
     * @param $boletos
     * @param Cnabpag $cnabpag
     * @param $caminho
     * @param bool $autorizaAgendamento
     * @throws \Exception
     */
    public static function geraCnabSicood(
        $boletos, Cnabpag $cnabpag, $caminho, $autorizaAgendamento
    ){
        $geraCnab = GeraCnabService::factory($cnabpag);
        $geraCnab->geraArquivo($caminho, $boletos, $autorizaAgendamento, $cnabpag->getIdopecnabpag());
    }

    /**
     * Gera CNAB Caixa
     * @param $boletos
     * @param Cnabpag $cnabpag
     * @param $caminho
     * @param bool $autorizaAgendamento
     * @throws \Exception
     */
    public static function geraCnabCaixa(
        $boletos, Cnabpag $cnabpag, $caminho, $autorizaAgendamento
    ){
        $geraCnab = GeraCnabService::factory($cnabpag);
        $geraCnab->geraArquivo($caminho, $boletos, $autorizaAgendamento, $cnabpag->getIdopecnabpag());
    }
    
    /**
     * Gera CNAB Bradesco
     * @param $boletos
     * @param Cnabpag $cnabpag
     * @param $caminho
     * @param bool $autorizaAgendamento
     * @throws \Exception
     */
    public static function geraCnabBradesco(
        $boletos, Cnabpag $cnabpag, $caminho, $autorizaAgendamento
    ){
        $geraCnab = GeraCnabService::factory($cnabpag);
        $geraCnab->geraArquivo($caminho, $boletos, $autorizaAgendamento, $cnabpag->getIdopecnabpag());
    }

    /**
     * Gera CNAB Brasil
     * @param $boletos
     * @param Cnabpag $cnabpag
     * @param $caminho
     * @param bool $autorizaAgendamento
     * @throws \Exception
     */
    public static function geraCnabBrasil(
        $boletos, Cnabpag $cnabpag, $caminho, $autorizaAgendamento
    ){
        $geraCnab = GeraCnabService::factory($cnabpag);
        $geraCnab->geraArquivo($caminho, $boletos, $autorizaAgendamento, $cnabpag->getIdopecnabpag());
    }

    /**
     * Exclui Título a Pagar da Remessa
     * @param $nrorg
     * @param $cdfilial
     * @param $cdoperador
     * @param $boletos
     * @throws \Exception
     */
    public function excluiTituloPagRemessa($nrorg, $cdfilial, $cdoperador, $boletos){
        $connection = $this->entityManager->getConnection();
        $connection->beginTransaction();
        try {

            // Percorre Todos os Boletos Marcados
            foreach ($boletos as $boleto){

                // Atualiza CNAB Boleto
                /** @var Boletpagar $boletpagar */
                $boletpagar = $this->remessaTituloFactory->buscaBoletpagar($nrorg, $boleto['CDFILIAL'], $boleto['NRLANBOLPAG']);

                $boletpagar->setNrcnabpagrem(null);
                $boletpagar->setIdsitucnabp('00');
                $this->entityManager->persist($boletpagar);

                // Atualiza CNAB
                /** @var Cnabpag $cnabpag */
                $cnabpag = $this->remessaTituloFactory->buscaCnabpag($nrorg, $boleto['CDEMPRESA'], $boleto['NRCNABPAGREM']);

                $cnabpag->setQtbolcnabpag($cnabpag->setQtbolcnabpag() - 1);
                $cnabpag->setVrtotalcnabpag($cnabpag->setVrtotalcnabpag() - $boleto['VRFINALBPAG']);
                $this->entityManager->persist($cnabpag);

                // Insere Log de Atualização Boleto Remessa Pagar
                /** @var Logboletorempag $logboletorempag */
                $logboletorempag = $this->remessaTituloFactory->buscaLogboletorempag(
                    $nrorg, $boleto['CDFILIAL'], $boleto['NRLANBOLPAG'], $boleto['CDEMPRESA'], $boleto['NRCNABPAGREM']
                );

                if(!$logboletorempag) {
                    $logboletorempag = $this->remessaTituloFactory->insereLogBoletoRem(
                        $nrorg, $boleto['CDFILIAL'], $boleto['NRLANBOLPAG'], $boleto['NRCNABPAGREM'], $boleto['CDEMPRESA'], $cdoperador
                    );
                }else $logboletorempag->setDtlogboletrpag(DateUtil::getDataAtual());
                $this->entityManager->persist($logboletorempag);

            }

            $this->entityManager->flush();
            $connection->commit();
        } catch (\Exception $e) {
            $connection->rollBack();
            throw $e;
        }
    }

    /**
     * Exclui Remessa Pagar
     * @param $nrorg
     * @param $cdfilial
     * @param $cdoperador
     * @param $boletos
     * @throws \Exception
     */
    public function excluiRemessaPag($nrorg, $cdfilial, $cdoperador, $boletos){
        $connection = $this->entityManager->getConnection();
        $connection->beginTransaction();
        try {
            
            // Percorre Todos os Boletos Marcados
            foreach ($boletos as $boleto) {

                // Atualiza CNAB Boleto
                /** @var Boletpagar $boletpagar */
                $boletpagar = $this->remessaTituloFactory->buscaBoletpagarRem($nrorg, $boleto['CDEMPRESA'], $boleto['NRCNABPAGREM']);
                
                if ($boletpagar) {
                    
                    /** @var Boletpagar $boletpag */
                    foreach ($boletpagar as $boletpag) {
                        $boletpag->setNrcnabpagrem(null);
                        $boletpag->setIdsitucnabp('00');
                        $this->entityManager->persist($boletpag);
                    }

                    // Atualiza CNAB
                    /** @var Cnabpag $cnabpag */
                    $cnabpag = $this->remessaTituloFactory->buscaCnabpag($nrorg, $boleto['CDEMPRESA'], $boleto['NRCNABPAGREM']);

                    $cnabpag->setQtbolcnabpag(0);
                    $cnabpag->setVrtotalcnabpag(0);
                    $this->entityManager->persist($cnabpag);

                    // Insere Log de Atualização Boleto Remessa Pagar
                    /** @var Logboletorempag $logboletorempag */
                    $logboletorempag = $this->remessaTituloFactory->buscaLogboletorempag(
                        $nrorg, $boleto['CDFILIAL'], $boleto['NRLANBOLPAG'], $boleto['CDEMPRESA'], $boleto['NRCNABPAGREM']
                    );

                    if (!$logboletorempag) {
                        $logboletorempag = $this->remessaTituloFactory->insereLogBoletoRem(
                            $nrorg, $boleto['CDFILIAL'], $boleto['NRLANBOLPAG'], $boleto['NRCNABPAGREM'], $boleto['CDEMPRESA'], $cdoperador
                        );
                    } else $logboletorempag->setDtlogboletrpag(DateUtil::getDataAtual());
                    $this->entityManager->persist($logboletorempag);
                }
            }

            $this->entityManager->flush();
            $connection->commit();
        } catch (\Exception $e) {
            $connection->rollBack();
            throw $e;
        }
    }

    /**
     * Gera Novo Código
     * @param $nrorg
     * @param $cdcontador
     * @return null|Novocodigo
     * @throws \Exception
     */
    public function geraNovoCodigo(
        $nrorg, $cdcontador
    ){
        try {
            /** @var Novocodigo $novocodigo */
            $novocodigo = $this->newCode->getCodeByTableName($cdcontador, $nrorg);
            return $novocodigo;
        } catch (\Exception $e) {
            throw $e;
            return null;
        }
    }

    /**
     * Formata Data Hora
     * @param $data
     * @return \DateTime|null|\Setforms2\Shared\ORM\DateTime|string
     */
    public static function formataDataHora($data)
    {
        if (!$data)
            return null;

        if (strlen($data) == 19)
            $data = DateUtil::truncateString($data);
        $data = DateUtil::getDataDeString($data);

        return $data;
    }
}