<?php

namespace Teknisa\FIN\Modules\MovimentacaoBancaria\Controllers;

use Teknisa\FIN\Modules\MovimentacaoBancaria\Services\Bco01400RecalculoLancamentoContaCorrenteService;
use Teknisa\FIN\Controllers\TeknisaCrud;
use Teknisa\Libs\Util\Masks\Masks;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DTO\Response\Message;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Exception;


class Bco01400RecalculoLancamentoContaCorrenteController extends TeknisaCrud
{
    
    protected $dataSourceName;
    protected $masks;
    protected $service;
    
    public function __construct($crudService, $parameterBag, Masks $masks, Bco01400RecalculoLancamentoContaCorrenteService $service) {   
        $this->service = $service;
        parent::__construct($crudService, $parameterBag);
        $this->masks = $masks;
    }
    
    /** 
     *  Essa função obrescreve a função find do teknisaCrud, com o objetivo de depois da
     *  busca no banco de data, usar a classe Masks para criar um campo novo para cada row
     *  que tenha os códigos de agência e conta correte formatados com suas respctivas 
     *  mascaras.
     *  
     *  @param $request vindo da requisição do frontend
     *  @param response que sera editado para retornar as rows para o frontend
     *  @return Rows do DataSet, do $response, com o campo de CDAGENCIA e CDCONTCORR
     *          formatados com a mascara, respectivamente CDAGENCIAMASK e CDCONTCORRMASK
     */
    public function findContCorrWithMasks(DTO\Request\Filter $request, DTO\Response $response)
    {
        $this->dataSourceName = 'recalculo_lanc_contcorr_contcorr';
        parent::find($request, $response);
        $dataSets = $response->getDataSets();
        
        if (!empty($dataSets)) {
            $rows = $dataSets[0]->getRows();
            for ($i = 0; count($rows) > $i; $i++) {
                $rows[$i]['CDAGENCIAMASK']  = $this->masks->mask($rows[$i]['CDAGENCIA'], $rows[$i]['CDPICTAGEN']);
                $rows[$i]['CDCONTCORRMASK'] = $this->masks->mask($rows[$i]['CDCONTCORR'], $rows[$i]['CDPICTCONT']);
            }
            
            $newDataSet = new DataSet($this->dataSourceName, $rows);
            $response->setDataSets(array(
                $newDataSet
            ));
        }
    }
    
    public function validateRow(DTO\Request\Row $request, DTO\Response $response) {
        
        $row = $request->getRow();
        try {
            $this->service->validateRow($row);
        } catch (\Exception $e){
            $variables = array();
            if ($e->getCode() == Bco01400RecalculoLancamentoContaCorrenteService::LANC_FORA_DO_PERIODO_CODE) {
                $lastResult = $this->service->getLastSelectContopectblResult();
                $variables['CODE'] = Bco01400RecalculoLancamentoContaCorrenteService::LANC_FORA_DO_PERIODO_CODE;
                $variables['DTINICCTBL'] = substr($lastResult['DTINICCTBL'], 0, -9);
                $variables['DTFINACCTBL'] = substr($lastResult['DTFINACCTBL'], 0, -9);
            }
            else {
                $variables['CODE'] = Bco01400RecalculoLancamentoContaCorrenteService::LANC_INVALIDO_CODE;
            }
            $message = new Message(null, null, null, $variables);
            $response->addMessage($message);
            $response->setStatus(Response::STATUS_ERROR);
        }
        
    }
    
    public function validateMultipleRows(DTO\Request\DataSet $request, DTO\Response $response) {
    
        $rows = $request->getDataSet()->getRows();
        
        $responseDataSource = new DataSet('rows_to_be_unchecked', $this->service->getRowsToBeUnchecked($rows));
        
        $response->addDataSet($responseDataSource);
        
    }
    
    public function recontabilizarLancamentos(DTO\Request\DataSet $request, DTO\Response $response) {
        
        $rows = $request->getDataSet()->getRows();
        try {
            $this->service->recontabilizarLancamentos($rows, $response);
        } catch(\Exception $e) {
            $response->setStatus(Response::STATUS_ERROR);
        }
        
    }
}